
ZoneClass = {}

borderdraw = {}
tracedraw = {}

function ZoneClass.new(w, h)
	local z = {}
	z.b = {}
	z.f = {}
	z.trace = {}
	z.WIDTH = w
	z.HEIGHT = h

	setmetatable(z, {__index=ZoneClass})
	-- create border
	for i = 1,h do
		table.insert(z.b,{x = 1, y = i})
	end
	for i = 2,w do
		table.insert(z.b,{x = i, y = h})
	end
	for i = h-1,1,-1 do
		table.insert(z.b,{x = w, y = i})
	end
	for i = w-1,2,-1 do
		table.insert(z.b,{x = i, y = 1})
	end

	-- create collision field
	table.insert(z.f,{1,1,w,h})

	return z
end

function toLine(points, mode)
	local index, line, onX
	line = {}
	onX = nil
	-- iterate points for directional change
	table.insert(line,points[1].x)
	table.insert(line,points[1].y)

	index = {x = points[1].x, y = points[1].y}

	for i = 2,#points do
		if index.x==points[i].x then
			if onX==false then
				table.insert(line,index.x)
				table.insert(line,index.y)
			end
			onX = true
		else
			if onX then
				table.insert(line,index.x)
				table.insert(line,index.y)
			end
			onX = false
		end
		-- push to next point
		index = {x = points[i].x, y = points[i].y}
	end
	if mode=="closed" then
		table.insert(line,points[1].x)
		table.insert(line,points[1].y)
	end
	return line
end

function ZoneClass.collision(x,y)
	local valid = false
	for i in pairs(z.f) do
		if x>=z.f[i][1] and x<=z.f[i][3] and y>=z.f[i][2] and y<=z.f[i][4] then valid = true end
	end
	return valid
end

function ZoneClass.colTrace(x,y)
	local valid = false
	if #z.trace>1 then
		for i = 1,#z.trace-1 do
			if x==z.trace[i].x and y==z.trace[i].y then valid = true end
		end
	end
	return valid
end

function ZoneClass.inField(x,y,field)
	local valid = false
	for i in pairs(field) do
		if x>=field[i][1] and x<=field[i][3] and y>=field[i][2] and y<=field[i][4] then valid = true end
	end
	return valid
end

function ZoneClass.updateBorder()
	-- fuse trace and border
	local b1, b2, inpos, outpos
	b1 = {}
	b2 = {}
	-- create two deep copies
	for i,_ in pairs(z.b) do
		table.insert(b1,{x = z.b[i].x, y = z.b[i].y})
		table.insert(b2,{x = z.b[i].x, y = z.b[i].y})
	end
	-- find in and out position
	for i,_ in pairs(b1) do
		if z.trace[1].x-1==b1[i].x and z.trace[1].y==b1[i].y
		or z.trace[1].x+1==b1[i].x and z.trace[1].y==b1[i].y
		or z.trace[1].y-1==b1[i].y and z.trace[1].x==b1[i].x
		or z.trace[1].y+1==b1[i].y and z.trace[1].x==b1[i].x
		then inpos = i end
		if z.trace[#z.trace].x-1==b1[i].x and z.trace[#z.trace].y==b1[i].y
		or z.trace[#z.trace].x+1==b1[i].x and z.trace[#z.trace].y==b1[i].y
		or z.trace[#z.trace].y-1==b1[i].y and z.trace[#z.trace].x==b1[i].x
		or z.trace[#z.trace].y+1==b1[i].y and z.trace[#z.trace].x==b1[i].x
		then outpos = i end
	end
	if inpos<outpos then
		for i = outpos-1,inpos+1,-1 do	table.remove(b1,inpos+1) end
		for i = outpos+1,#b2 do	table.remove(b2,outpos+1) end
		for i = 1, inpos-1 do table.remove(b2,1) end
		for i = #z.trace,1,-1 do table.insert(b1,inpos+1,z.trace[i]) end
		for i = #z.trace,1,-1 do table.insert(b2,z.trace[i]) end
	elseif inpos>outpos then
		for i = inpos-1,outpos+1,-1 do table.remove(b1,outpos+1) end
		for i = inpos+1,#b2 do table.remove(b2,inpos+1) end
		for i = 1, outpos-1 do table.remove(b2,1) end
		for i = 1,#z.trace do table.insert(b1,outpos+1,z.trace[i]) end
		for i = 1,#z.trace do table.insert(b2,z.trace[i]) end
	end
	return b1,b2
end

function ZoneClass.updateFields(border)
	local t = {}
	local l = {}
	local f = {}
	local fx,fy
	fx = border[1].x
	fy = border[1].y
	--sort by cols and rows	
	for i,_ in pairs(border) do
		if i>1 and i<#border then
			if 	border[i].x==border[i+1].x and border[i].y-1==border[i+1].y
			and border[i].y==border[i-1].y and border[i].x-1==border[i-1].x then
				fx = border[i].x
				fy = border[i].y
			end
		end
		k = "x"..border[i].x.."y"..border[i].y
		t[k] = 1
	end

	for i = 1,z.WIDTH do
		for j = 1,z.HEIGHT do
			k = "x"..i.."y"..j
			if not t[k] then t[k]=0 end
		end
	end

	-- print out t
--	for j = 1,z.HEIGHT do
--		for i = 1,z.WIDTH do
--			k = "x"..i.."y"..j
--			if t[k]==1 then
--				table.insert(l,"1")
--			else
--				table.insert(l,"0")
--			end
--		end
--		print(table.concat(l," "))
--	l = {}
--	end
--	print()
	
	-- scanline/queue fill
	local k,l_k, r_k, x, y, lx, ly, rx, ry
	local queue = {}
	local not_empty = true

	-- fillpoint
	x,y = fx-1,fy-1
	print("fillpoint ",x,y)
	k = "x"..x.."y"..y
	while not_empty do
		--check, pass on neighbours, fill & kill key
		if t[k]==0 then
			x = string.sub(k,2,(string.find(k,"y")-1))
			y = string.sub(k,(string.find(k,"y")+1),#k)
			l_k = "x"..x.."y"..y
			r_k = "x"..(x+1).."y"..y
			--fill & kill neighbours while queueing empties
			while t[l_k]==0 do
				lx = string.sub(l_k,2,(string.find(l_k,"y")-1))
				ly = string.sub(l_k,(string.find(l_k,"y")+1),#l_k)
				t[l_k] = 1
				queue[l_k] = 0
				if t["x"..lx.."y"..(ly-1)]==0 then queue["x"..lx.."y"..(ly-1)] = 1 end
				if t["x"..lx.."y"..(ly+1)]==0 then queue["x"..lx.."y"..(ly+1)] = 1 end
				l_k = "x"..(lx-1).."y"..ly
			end
			while t[r_k]==0 do
				rx = string.sub(r_k,2,(string.find(r_k,"y")-1))
				ry = string.sub(r_k,(string.find(r_k,"y")+1),#r_k)
				t[r_k] = 1
				queue[r_k] = 0
				if t["x"..(rx+1).."y"..(ry-1)]==0 then queue["x"..(rx+1).."y"..(ry-1)] = 1 end
				if t["x"..(rx+1).."y"..(ry+1)]==0 then queue["x"..(rx+1).."y"..(ry+1)] = 1 end
				r_k = "x"..(rx+1).."y"..ry
			end
			--count queue and turn last item to key
			not_empty = false
			for i,_ in pairs(queue) do
				if queue[i]==1 then
					k = i
					queue[i] = 0
					not_empty = true
					break
				end
			end
		end
	end
	
	local col,c,prev
	col = {}
	c = {}
	-- fill c with endpoints only
	for i = 1,z.WIDTH do
		c[i] = {}
		for j = 1,z.HEIGHT do
			k = "x"..i.."y"..j
			if t[k]==1 then table.insert(col,j) end
		end
		table.insert(c[i],col[1])
		for k = 2,#col do
			if col[k]-1~=col[k-1] then
				table.insert(c[i],col[k-1])
				table.insert(c[i],col[k])
			end
		end
		table.insert(c[i],col[#col])

		-- compare with neighbor column
		if c[i][1]~=nil then
			if #f==0 then
				for l = 1,#c[i],2 do
					table.insert(f,{i,c[i][l],i,c[i][l+1]})
				end
			else
				--check if columns are identical
				same = true
				if #c[i]==#c[i-1] then
					for l in pairs(c[i]) do
						if c[i-1][l]~=c[i][l] then same = false end
					end
				else same = false end
				prev = #f
				if same then
					for l = 1,#c[i],2 do
						f[prev][3] = i
						prev = prev-1
					end
				else
					for l = 1,#c[i],2 do
						table.insert(f,{i,c[i][l],i,c[i][l+1]})
					end
				end
			end
		end
	col = {}
	end
	print("fields:")
	for i in pairs(f) do
		print(i,table.concat(f[i],","))
	end
	return f
end


function ZoneClass.onBorder(x,y)
	local valid = false
	local rnd_x,rnd_y
	local tb1,tb2,tf1,tf2
	rnd_x = math.floor(x+0.5)
	rnd_y = math.floor(y+0.5)
	-- check border points
	for i,_ in pairs(z.b) do
		if rnd_x==z.b[i].x and rnd_y==z.b[i].y then	valid = true end
	end
	-- back on border: clear trace and connect to border
	if valid and #z.trace>0 then
		tb1,tb2 = z.updateBorder()
		tf1,tf2 = z.updateFields(tb1), z.updateFields(tb2)
		if z.inField(s[1].x,s[1].y,tf1) then
			print("choosing field1")
			z.b = tb1
			z.f = tf1
		else
			print("choosing field2")
			z.b = tb2
			z.f = tf2
		end
		borderdraw = toLine(z.b,"closed")
		z.trace = {}
		tracedraw = {}
	end
	-- in field: trace
	if not valid then
		table.insert(z.trace,{x = rnd_x, y = rnd_y})
		for i,_ in pairs(z.trace) do
			if i<#z.trace and z.trace[i].x==z.trace[i+1].x and z.trace[i].y==z.trace[i+1].y then table.remove(z.trace,i+1) end
		end
		tracedraw = toLine(z.trace)
		table.insert(tracedraw,rnd_x)
		table.insert(tracedraw,rnd_y)
	end
	return valid
end

function ZoneClass.update(x,y,dir)
end

function ZoneClass.draw()
	if #z.f>0 then
		for i in pairs(z.f) do
			love.graphics.setColor(120,140, 0,i*70)
			love.graphics.rectangle("fill",z.f[i][1],z.f[i][2],z.f[i][3]-z.f[i][1]+1,z.f[i][4]-z.f[i][2])
		end
	end
	love.graphics.setColor(255, 255, 255)
	love.graphics.line(borderdraw)
	if #tracedraw>3 then
		love.graphics.setColor(0, 255, 255)
		love.graphics.line(tracedraw)
	end
end











